from easyansi.core.color_builder import ColorBuilder as _ColorBuilder
_builder = _ColorBuilder.PALETTE_16._get_color_code_by_int

DEFAULT_IDX = -1
DEFAULT_FG = _builder(DEFAULT_IDX, False)
DEFAULT_BG = _builder(DEFAULT_IDX, True)
DEFAULT = DEFAULT_FG

BLACK_IDX = 0
BLACK_FG = _builder(BLACK_IDX, False)
BLACK_BG = _builder(BLACK_IDX, True)
BLACK = BLACK_FG

RED_IDX = 1
RED_FG = _builder(RED_IDX, False)
RED_BG = _builder(RED_IDX, True)
RED = RED_FG

GREEN_IDX = 2
GREEN_FG = _builder(GREEN_IDX, False)
GREEN_BG = _builder(GREEN_IDX, True)
GREEN = GREEN_FG

YELLOW_IDX = 3
YELLOW_FG = _builder(YELLOW_IDX, False)
YELLOW_BG = _builder(YELLOW_IDX, True)
YELLOW = YELLOW_FG

BLUE_IDX = 4
BLUE_FG = _builder(BLUE_IDX, False)
BLUE_BG = _builder(BLUE_IDX, True)
BLUE = BLUE_FG

MAGENTA_IDX = 5
MAGENTA_FG = _builder(MAGENTA_IDX, False)
MAGENTA_BG = _builder(MAGENTA_IDX, True)
MAGENTA = MAGENTA_FG

CYAN_IDX = 6
CYAN_FG = _builder(CYAN_IDX, False)
CYAN_BG = _builder(CYAN_IDX, True)
CYAN = CYAN_FG

WHITE_IDX = 7
WHITE_FG = _builder(WHITE_IDX, False)
WHITE_BG = _builder(WHITE_IDX, True)
WHITE = WHITE_FG

BRIGHT_BLACK_IDX = 8
BRIGHT_BLACK_FG = _builder(BRIGHT_BLACK_IDX, False)
BRIGHT_BLACK_BG = _builder(BRIGHT_BLACK_IDX, True)
BRIGHT_BLACK = BRIGHT_BLACK_FG

BRIGHT_RED_IDX = 9
BRIGHT_RED_FG = _builder(BRIGHT_RED_IDX, False)
BRIGHT_RED_BG = _builder(BRIGHT_RED_IDX, True)
BRIGHT_RED = BRIGHT_RED_FG

BRIGHT_GREEN_IDX = 10
BRIGHT_GREEN_FG = _builder(BRIGHT_GREEN_IDX, False)
BRIGHT_GREEN_BG = _builder(BRIGHT_GREEN_IDX, True)
BRIGHT_GREEN = BRIGHT_GREEN_FG

BRIGHT_YELLOW_IDX = 11
BRIGHT_YELLOW_FG = _builder(BRIGHT_YELLOW_IDX, False)
BRIGHT_YELLOW_BG = _builder(BRIGHT_YELLOW_IDX, True)
BRIGHT_YELLOW = BRIGHT_YELLOW_FG

BRIGHT_BLUE_IDX = 12
BRIGHT_BLUE_FG = _builder(BRIGHT_BLUE_IDX, False)
BRIGHT_BLUE_BG = _builder(BRIGHT_BLUE_IDX, True)
BRIGHT_BLUE = BRIGHT_BLUE_FG

BRIGHT_MAGENTA_IDX = 13
BRIGHT_MAGENTA_FG = _builder(BRIGHT_MAGENTA_IDX, False)
BRIGHT_MAGENTA_BG = _builder(BRIGHT_MAGENTA_IDX, True)
BRIGHT_MAGENTA = BRIGHT_MAGENTA_FG

BRIGHT_CYAN_IDX = 14
BRIGHT_CYAN_FG = _builder(BRIGHT_CYAN_IDX, False)
BRIGHT_CYAN_BG = _builder(BRIGHT_CYAN_IDX, True)
BRIGHT_CYAN = BRIGHT_CYAN_FG

BRIGHT_WHITE_IDX = 15
BRIGHT_WHITE_FG = _builder(BRIGHT_WHITE_IDX, False)
BRIGHT_WHITE_BG = _builder(BRIGHT_WHITE_IDX, True)
BRIGHT_WHITE = BRIGHT_WHITE_FG
